require 'dry-container'

module EdgycircleToolbox
  module CQRS
    class CommandBuilder
      class DecoratorContainer
        include Dry::Container::Mixin

        configure do |config|
          config.registry = ->(container, key, item, options) { (container[key] ||= []).push(item) }
          config.resolver = ->(container, key) { container[key] || [] }
        end
      end

      def initialize
        @command_container = Dry::Container.new
        @decorator_container = DecoratorContainer.new
      end

      def register(command_class)
        key = command_class.name.split("::").last.gsub("Command", "")
        @command_container.register(key, command_class)
      end

      def decorate(command_class, decorator_class)
        @decorator_container.register(command_class, decorator_class)
      end

      def build(valid_parameters, context = nil)
        command_class = @command_container.resolve(valid_parameters[:name])
        decorator_classes = @decorator_container.resolve(command_class)
        command = command_class.new(valid_parameters)
        decorator_classes.inject(command) { |command, decorator_class| decorator_class.new(command, context) }
      end

      def validate(original_parameters)
        command_class = @command_container.resolve(original_parameters[:name])
        command_class.schema.call(original_parameters)
      end
    end
  end
end
