require 'edgycircle_toolbox/sonapi/blueprint'
require 'edgycircle_toolbox/sonapi/serializer'
require 'edgycircle_toolbox/sonapi/deserializer'

module EdgycircleToolbox
  module Sonapi
    module Resource
      module ClassMethods
        def type(type)
          @blueprint = Blueprint.new(type)
        end

        def serialize_resource(object)
          Serializer.new(@blueprint).call(object)
        end

        def serialize(object, options = {})
          if object.respond_to?(:map)
            data = object.map { |item| serialize_resource(item) }
          else
            data = serialize_resource(object)
          end

          result = {
            "data" => data
          }

          if options[:included_resources]
            result.merge!({ "included" => options[:included_resources] })
          end

          result
        end

        def deserialize(parameters)
          Deserializer.new(@blueprint).call(parameters)
        end

        def attribute(attribute_name)
          @blueprint.add_attribute attribute_name.to_s
        end

        def dynamic_attributes(callable)
          @blueprint.dynamic_attributes = callable
        end

        def parameter_filter(callable)
          @blueprint.parameter_filter = callable
        end
      end

      def self.included(base)
        base.extend(ClassMethods)
      end
    end
  end
end
